
from gi.repository import Gtk

from . import loop
from . import record
from . import sets
from . import play
from . import build

def min(b,win):
	win.minimize()
	
def cl(b,d):
	loop.stop()

input=0x1F399

def init(combo):
	global box
	box=Gtk.Box()
	box.append(sets.colorButton(chr(input), record.start, "Record audio", input))
	box.append(sets.colorButton(chr(0x2699), sets.sets, "Settings", combo))
	box.append(sets.colorButton("_", min, "Minimize", combo[0]))
	box.append(sets.colorButton("X", cl, "Exit"))
	box.append(play.entry)
	box.append(play.openbutton)
	box.append(play.button)
	box.append(build.init())
	return box



import pyaudio
import wave

from time import time

from . import sets

wavefile=None

def callback(in_data, frame_count, time_info, status):
	wavefile.writeframes(in_data)
	return (None, pyaudio.paContinue)

def start(b,ready):
	global audio,stream,wavefile
	if not wavefile:
		audio = pyaudio.PyAudio() # create pyaudio instantiation
		chans=1
		bits=pyaudio.paInt16 # 16-bit resolution
		rate=16000 # 16kHz sampling rate
		# create pyaudio stream
		stream = audio.open(format = bits,rate = rate,channels = chans,
			input = True,
			frames_per_buffer=rate, # one second of samples
			stream_callback=callback)
		w=sets.get_data_file(str(int(time()))+".wav")
		wavefile = wave.open(w,'wb')
		wavefile.setnchannels(chans)
		wavefile.setsampwidth(audio.get_sample_size(bits))
		wavefile.setframerate(rate)
		#visual
		b._set_text_(chr(0x23F9))
		print(w)
		return
	stop()
	b._set_text_(chr(ready))
def stop():
	# stop the stream, close it, terminate the pyaudio instantiation
	stream.stop_stream()
	stream.close()
	audio.terminate()
	# close the file
	global wavefile
	wavefile.close()
	wavefile=None
def terminate():
	if wavefile:
		stop()



from . import point
from . import draw
from . import graph
from . import drawscroll
from . import points
from . import level

def autodrag(p,x,y):
	p._dend_(x,y)
	graph.area.queue_draw()

def coord(p,h):
	hg=p._height_
	p._height_=h
	x,y=p._coord_(draw.wstore,draw.hstore)
	p._height_=hg
	return x,y

def base(b,d):
	p=point.lastselect
	x,y=coord(p,0)
	autodrag(p,x,y)

#tests: 3 tests on special way

def left(b,d):
	p=point.lastselect
	if p.get_parent(): #only if in current cont, else will be an error, visible and gtk error
		_,y=draw.cont.get_child_position(p)
		y+=point.const
		if drawscroll.landscape:
			ix=points.points.index(p)
			if ix==0 or points.points[ix-1]._offset_<draw.offset:
				autodrag(p,0,y)
			else:
				autodrag(p,points.points[ix-1]._offset_,y)
		else:
			autodrag(p,0,y)

def right(b,d):
	p=point.lastselect
	if p.get_parent():
		_,y=draw.cont.get_child_position(p)
		y+=point.const
		if drawscroll.landscape:
			ix=points.points.index(p)
			sz=len(points.points)
			if ix+1==sz or points.points[ix+1]._offset_>(draw.offset+draw.size):
				autodrag(p,draw.size,y)
			else:
				autodrag(p,points.points[ix+1]._offset_,y)
		else:
			x,_=coord(p,level.get_size()-1)
			autodrag(p,x,y)

def up(b,d):
	p=point.lastselect
	if p.get_parent():
		x,_=draw.cont.get_child_position(p)
		x+=point.const
		if drawscroll.landscape:
			autodrag(p,x,0)
		else:
			ix=points.points.index(p)
			if ix==0 or points.points[ix-1]._offset_<draw.offset:
				autodrag(p,x,0)
			else:
				autodrag(p,x,points.points[ix-1]._offset_)

def down(b,d):
	p=point.lastselect
	if p.get_parent():
		x,_=draw.cont.get_child_position(p)
		x+=point.const
		if drawscroll.landscape:
			_,y=coord(p,level.get_size()-1) #simple get height will fall to 256 ([0,255] are the allowed heights)
			autodrag(p,x,y)
		else:
			ix=points.points.index(p)
			sz=len(points.points)
			if ix+1==sz or points.points[ix+1]._offset_>(draw.offset+draw.size):
				autodrag(p,x,draw.size)
			else:
				autodrag(p,x,points.points[ix+1]._offset_)




from gi.repository import Gtk

from . import sets
from . import draw
from . import seloff
from . import drawscroll

class inttext(sets.colorLabel):
	def _get_(self):
		return int(self.get_text())
	def _set_(self,t):
		self._set_text_(str(t))

def init():
	global atleft,atright
	b=Gtk.Box(homogeneous=True)
	atleft=inttext("0")
	atleft.set_halign(Gtk.Align.START)
	atright=inttext("0")
	atright.set_halign(Gtk.Align.END)
	b.append(atleft)
	b.append(seloff.init())
	b.append(atright)
	return b

def cnged(a,visible):
	val=int(a.get_value())
	l=draw.offset+val
	atleft._set_text_(str(l))
	r=max(draw.length-visible-l,0)
	atright._set_text_(str(r))
def cged(a):
	cnged(a,int(a.get_page_size()))
def cgd(a,d):
	cged(a)
	if sets.turn_page.get_active():
		drawscroll.edge(a.get_value(),a.get_upper()-a.get_page_size())

def calculate(pos):
	#pos is is relative to draw
	pos+=draw.offset
	if pos>draw.length: #here and 2 more places (about samples length vs points length)
		pos=draw.length
	st=seloff.start._get_()
	en=seloff.end._get_()
	if st==0 and en==0:
		seloff.start._set_(pos)
		seloff.end._set_(pos)
		return
	n=abs(st-pos)
	m=abs(en-pos)
	if m<n or (m==n and en<pos):
		if en<pos:
			draw.sel(en,pos)
		else:
			draw.unsel(pos,en)
		seloff.end._set_(pos)
	else:
		if pos<st:
			draw.sel(pos,st)
		else:
			draw.unsel(st,pos)
		seloff.start._set_(pos)
	draw.area.queue_draw()




import pyaudio
import wave

from gi.repository import GLib

from . import sets
from . import draw
from . import seloff
from . import points
from . import build
from . import forms
from . import drawscroll
from . import error

wavefile=None
output=0x25B6
#23F5
timer=0

def activate(wd,d):
	terminate()
	launch()
def toggle(b,d):
	if not wavefile:
		if launch():
			start()
		return
	if stream.is_stopped():
		start()
	else:
		pause()

def init():
	global entry,button,openbutton
	entry=sets.colorEntry()
	openbutton=sets.colorButton(chr(0x1f4d6), activate, "Open")
	button=sets.colorButton(chr(output), toggle, "Play")
	entry.connect('activate',activate,None)

def callback(in_data, frame_count, time_info, status):
	data = wavefile.readframes(frame_count)
	return (data, pyaudio.paContinue)
def wave_open(f):
	global wavefile
	wavefile=wave.open(f,'rb')
def waveopen(f):
	wave_open(f)
	build.button.set_sensitive(False)

def launch():
	try:
		f=entry.get_text()
		waveopen(f)
		sampwidth=wavefile.getsampwidth()
		rate = wavefile.getframerate()
		channels = wavefile.getnchannels()
		draw.length=wavefile.getnframes()
		data = wavefile.readframes(draw.length)
		wavefile.rewind()#for playing
		#pyaudio/draw/bar
		open(sampwidth,channels,rate)
		#points
		points.read(f,False)
		#samples from file
		draw.get_samples(sampwidth,channels,data)
		#only if size is less than the screen
		forms.drawpoints(drawscroll.win.get_width(),drawscroll.win.get_height())
	except Exception as e:
		error.open(e.__str__())
		return False
	return True
def open(sampwidth,channels,rate):
	global audio,stream
	# create pyaudio stream
	audio = pyaudio.PyAudio() # create pyaudio instantiation
	stream = audio.open(format=audio.get_format_from_width(sampwidth),rate=rate,channels=channels,
		output = True,start=False,stream_callback=callback)
	#open
	draw.open(sampwidth,channels)
	seloff.open()
def start():
	stream.start_stream()
	button._set_text_(chr(0x23F8))
	global timer
	timer=GLib.timeout_add_seconds(1,is_act,None)
def pausing():
	stream.stop_stream()
	button._set_text_(chr(output))
def pause():
	pausing()
	global timer
	if timer:
		GLib.source_remove(timer)
		timer=0
def stop():
	# stop the stream, close it, terminate the pyaudio instantiation
	pause()
	stream.close()
	audio.terminate()
	# close the file
	global wavefile
	wavefile.close()
	wavefile=None
	build.button.set_sensitive(True)
def terminate():
	if wavefile:
		seloff.stop.emit(sets._click_)

def is_act(d):
	if not stream.is_active():
		pausing()
		global timer
		timer=0
		wavefile.rewind()
		return False
	return True

formats={pyaudio.paInt16:'h',pyaudio.paUInt8:'B',pyaudio.paInt8:'b',
	pyaudio.paFloat32:'f',pyaudio.paInt32:'i'}
def scan(sampwidth,channels):
	fm=scan_format(sampwidth,channels)
	return '<'+fm*channels
def scan_format(sampwidth,channels):
	return formats[audio.get_format_from_width(sampwidth)]

def save_file(f_in,s,c,r):
	with wave.open(f_in,'wb') as file:
		file.setsampwidth(s)
		file.setnchannels(c)
		file.setframerate(r)
		#.setparams((1, 4, Fs, 0, 'NONE', 'not compressed'))
		sc=scan(s,c)
		b=b"".join((wave.struct.pack(sc,i) for i in draw.samples))
		file.writeframes(b)#this is setting nframes, oposite of writeframesraw
def save_opened(f_in):
	save_file(f_in,wavefile.getsampwidth(),wavefile.getnchannels(),wavefile.getframerate())
def save(b,d):
	f_in=entry.get_text()
	save_opened(f_in)
	points.write(f_in)
def saveshort(b,d):
	points.write(entry.get_text())




import os

from . import play
from . import points

def file():
	global temp
	temp=loc()
	play.save_opened(temp)
	play.wavefile.close()
	play.wave_open(temp)

def loc():
	return points.fpath(play.entry.get_text(),"temp")

def close():
	#temp=loc()
	try:  #can change the entry and temp need to be the last one, but then can be nothing, then use try
		if os.path.exists(temp):
			os.remove(temp)
	except:
		pass




from gi.repository import Gtk

from . import info
from . import sets
from . import drawscroll
from . import draw
from . import level
from . import r_offset

def open(b,d):
	box=Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
	bx=Gtk.Box()
	cur,max=get_vals()
	bx.append(sets.colorLabel(cur.__str__()+" 0-"))
	x=max.__str__()
	maxlab=sets.colorLabel(x)
	bx.append(maxlab)
	buf=Gtk.EntryBuffer()
	bx.append(sets.colorEntry(buf))
	bx.append(sets.colorButton("Go",go,"Proceed",[max,buf]))
	box.append(bx)
	box.append(sets.colorButton("First",callback,"Start",0))
	a=0 if cur==0 else (cur-1)
	box.append(sets.colorButton("Previous",callback,a.__str__(),a))
	a=x if cur==max else (cur+1)
	box.append(sets.colorButton("Next",callback,a.__str__(),a))
	box.append(sets.colorButton("Last",callback,"End",max))
	box.append(sets.colorButton("Cancel",cancel,"Abort"))
	info.win.set_child(box)

def cancel(b,combo):
	finish()
def finish():
	info.win.set_child(info.box)
def done(a):
	draw.offset=page()*a
	finish()
	r_offset.cged(adjust())
	draw.redraw()

def callback(b,a):
	done(a)

def page():
	if drawscroll.landscape:
		return drawscroll.win.get_width()*drawscroll.factor
	return drawscroll.win.get_width()*drawscroll.factor
def adjust():
	if drawscroll.landscape:
		return drawscroll.win.get_hadjustment()
	return drawscroll.win.get_vadjustment()

def get_vals():
	of=draw.offset
	p=page()
	cur=int(of/p)
	max=int(draw.length/p)
	return (cur,max)

def go(b,d):
	max,buf=d
	a=buf.get_text()
	if a.isdigit():
		b=int(a)
		if b>max:
			buf.set_text(max.__str__(),-1)
			return
		done(b)
		return
	level.not_a_digit(buf)



import wave

from gi.repository import Gtk,Gdk
import cairo

from . import sets
from . import drawscroll
from . import play
from . import seloff
from . import forms
from . import graph

#area,cont,over
offset=0
#length
#samples
#size
#size here is visible samples

#sampsize,baseline,surface,ostore,wstore,hstore

def draw_none(widget,cr,width,height,d,u):
	co=Gdk.RGBA()
	if co.parse(sets.get_text_color()):
		#set line color
		cr.set_source_rgb(co.red,co.green,co.blue)
	cr.set_line_width(1)#default 2.0; cairo scale is 1
	if width>=height:
		y=height/2
		cr.move_to(0,y)
		cr.line_to(width,y)
	else:
		x=width/2
		cr.move_to(x,0)
		cr.line_to(x,height)
	cr.stroke()
def draw_cont(widget,cr,width,height,d,d2):
	n=length-offset
	if drawscroll.calculate(n):
		return
	global ostore,wstore,hstore
	if ostore!=offset or wstore!=width or hstore!=height:
		global size
		ostore=offset
		wstore=width
		hstore=height
		size=min(width,n) if drawscroll.landscape else min(height,n)
		unsel(offset,offset+size)
		draw_sel()
	cr.set_source_surface (surface, 0, 0)
	cr.paint ()
def draw_sel():
	start=seloff.start._get_()
	end=seloff.end._get_()
	if start<(offset+size) and end>offset:
		sel(max(start,offset),min(end,offset+size))
def surf():
	global surface
	surface = area.get_native().get_surface().create_similar_surface(cairo.Content.COLOR,wstore,hstore)
def redraw():
	surf()
	graph.redraw()
	area.queue_draw()

def init():
	global area,over
	area=Gtk.DrawingArea()
	area.set_draw_func (draw_none,None,None)
	over=Gtk.Overlay()
	over.set_child(area)
	return over
def close():
	over.remove_overlay(cont)
	graph.close(over)
	global offset,length#for r_offset
	offset=0
	length=0
	area.disconnect(res_id)
	area.set_draw_func (draw_none,None,None)
	drawscroll.calculate(0)
	play.stop()
def get_samples(sampwidth,channels,data):
	blockAlign=sampwidth*channels
	tot=length*blockAlign
	global samples
	samples=[]
	scan=play.scan(sampwidth,channels)
	for i in range(0, tot, blockAlign):
		s=wave.struct.unpack(scan, data[i:i+blockAlign])
		samples.append(s[0])
def open(sampwidth,channels):
	graph.open(over)
	global cont
	cont=Gtk.Fixed()#fixed is not tracking window default-width
	over.add_overlay(cont)
	#
	global wstore,hstore,sampsize,baseline
	reset()
	#wstore=-1 one flag is enaugh
	#hstore=-1
	sampsize=2**(8*sampwidth)
	fm=play.scan_format(sampwidth,channels)
	baseline=(1/2) if fm.islower() else 0
	global res_id
	res_id=area.connect_after ("resize", resize_cb, None)
	area.set_draw_func (draw_cont,None,None)
	#need landscape,there is a case when n<length when no surface, and points at reopen
	area.emit("resize",area.get_width(),area.get_height())

def resize_cb(a,w,h,d):
	drawscroll.set_landscape()
	global surface
	surface = a.get_native().get_surface().create_similar_surface(cairo.Content.COLOR,w,h)
	graph.surf(w,h)
	forms.redraw(w,h)

def paintland(cr,y,ratio,a,b):
	for i in range(a,b):
		j=i-offset
		cr.move_to(j,y)
		z=samples[i]
		r=ratio*z+y
		cr.line_to(j,r)
def paintport(cr,x,ratio,a,b):
	for i in range(a,b):
		j=i-offset
		cr.move_to(x,j)
		z=samples[i]
		c=ratio*z+x
		cr.line_to(c,j)
def sel(a,b):
	paint(a,b,sets.get_fgcolor())
def unsel(a,b):
	paint(a,b,sets.get_color())
def paint(a,b,clr):
	cr=cairo.Context(surface)
	cr.set_line_width(1)#this at start?nothing
	co=Gdk.RGBA()
	if co.parse(clr):
		cr.set_source_rgb(co.red,co.green,co.blue)
	if drawscroll.landscape:
		paintland(cr,hstore*baseline,hstore/sampsize,a,b)
	else:
		paintport(cr,wstore*baseline,wstore/sampsize,a,b)
	cr.stroke()

def reset():
	global ostore
	ostore=-1




from gi.repository import Gtk,Gdk
import cairo
import math

from . import points
from . import sets
from . import drawscroll
from . import arc
from . import draw
from . import forms

def open(ovr):
	global area
	area=Gtk.DrawingArea()
	area.set_draw_func(draw_cont,None,None)
	ovr.add_overlay(area)
def close(ovr):
	ovr.remove_overlay(area)
def draw_cont(widget,cr,width,height,d,d2):
	cr.set_source_surface(surface, 0, 0)
	cr.paint()
def surf(w,h):
	global surface
	surface = area.get_native().get_surface().create_similar_surface(cairo.Content.COLOR_ALPHA,w,h)

def put(ix,c1,w,h):
	if ix>0:
		c0=points.points[ix-1]
		line(c0,c1,w,h)
	elif len(points.points)>1:
		c0=points.points[1]
		line(c1,c0,w,h)
def line(c0,c1,w,h):
	cr=cairo.Context(surface)
	co=Gdk.RGBA()
	if co.parse(sets.get_fgcolor2()):
		cr.set_source_rgb(co.red,co.green,co.blue)
	line_draw(cr,c0,c1,w,h)
def line_draw(cr,c0,c1,w,h):
	a0=c0._coord_(w,h)
	a1=c1._coord_(w,h)
	if c0._inter_ or c1._inter_:
		arc.draw(cr,a0[0],a0[1],a1[0],a1[1],c0._concav_)
	else:
		#don't let line width corners to intersect
		p0,p1=coords(a0[0],a0[1],a1[0],a1[1])
		cr.move_to(p0[0],p0[1])
		cr.line_to(p1[0],p1[1])
	cr.stroke()
def lines(dels,puts,w,h):
	cr=cairo.Context(surface)
	ope=cr.get_operator()
	cr.set_operator(cairo.Operator.CLEAR)
	for d in dels:
		clearline(cr,d[0],d[1],w,h)
	cr.set_operator(ope)
	co=Gdk.RGBA()
	if co.parse(sets.get_fgcolor2()):
		cr.set_source_rgb(co.red,co.green,co.blue)
	for p in puts:
		line_draw(cr,p[0],p[1],w,h)
def dels(ds,w,h):
	cr=cairo.Context(surface)
	cr.set_operator(cairo.Operator.CLEAR)
	for d in ds:
		clearline(cr,d[0],d[1],w,h)
def take(ix,pnt):
	sz=len(points.points)
	if ix>0:
		d=[[points.points[ix-1],pnt]]
		if ix+1<sz:
			d.append([pnt,points.points[ix+1]])
		return d
	elif sz>1:
		c0=points.points[1]
		return [[pnt,c0]]
	return None
def clearline(cr,a0,a1,w,h):
	c0=a0._coord_(w,h)
	c1=a1._coord_(w,h)
	#arc is lineing to circle start, not convenient
	if a0._inter_ or a1._inter_:
		arc.draw(cr,c0[0],c0[1],c1[0],c1[1],a0._concav_)
		lw=cr.get_line_width()
		cr.set_line_width(lw+1)  #only this here
		cr.stroke()
		cr.set_line_width(lw)
		return
	x,y=coords0(c0[0],c0[1],c1[0],c1[1],1)   #it's tested
	p0=[c0[0]+x,c0[1]+y]
	p1=[c1[0]-x,c1[1]-y]
	h=cr.get_line_width()/2+1   #it's tested
	x,y=xy_h(h,x,y)
	cr.move_to(p0[0]-x,p0[1]+y)
	cr.line_to(p0[0]+x,p0[1]-y)
	#
	cr.line_to(p1[0]+x,p1[1]-y)
	cr.line_to(p1[0]-x,p1[1]+y)
	#
	cr.line_to(p0[0]-x,p0[1]+y)
	cr.fill()

def coords0(x0,y0,x1,y1,extra=0):
	x=x1-x0
	y=y1-y0
	#l=point.const-extra there was a problem at arc at same thing with this
	l=-extra
	if drawscroll.landscape:
		r=rads(y,x)
		x=math.cos(r)*l
		y=math.sin(r)*l
	else:
		r=rads(x,y)
		x=math.sin(r)*l
		y=math.cos(r)*l
	return (x,y)
def coords(x0,y0,x1,y1):
	x,y=coords0(x0,y0,x1,y1)
	return ([x0+x,y0+y],[x1-x,y1-y])
def xy_h(h,x,y):
	if drawscroll.landscape:
		r=math.atan2(y,x)
		return (math.sin(r)*h,math.cos(r)*h)
	r=math.atan2(x,y)
	return (math.cos(r)*h,math.sin(r)*h)
def rads(a,b):
#this is atan2
	t=a/b if b else math.inf
	return math.atan(t)

def redraw():
	surf(draw.wstore,draw.hstore)
	forms.redraw(draw.wstore,draw.hstore)
	area.queue_draw()



from gi.repository import Gtk,Gdk

from . import draw
from . import r_offset
from . import seloff
from . import sets
from . import point
from . import step

#size,landscape,win
#size here is "f" times
factor=2

def set_landscape():
	global landscape
	landscape=win.get_width()>=win.get_height()
def calculate(n):
	w=win.get_width()
	h=win.get_height()
	d=draw.area
	global size
	if landscape:
		#30000 maximum size of an X window
		if n>(size:=(w*factor)):
			n=size
		elif n<=w:#cannot go back at overshot/scroll without this
			if draw.length>w:
				n=w+1
			else:
				n=w
		if d.get_width()!=n or d.get_height()!=h:
			d.set_size_request(n,h)
			r_offset.cnged(win.get_hadjustment(),w)
			return True
	else:
		if n>(size:=(h*factor)):
			n=size
		elif n<=h:
			if draw.length>h:
				n=h+1
			else:
				n=h
		if d.get_width()!=w or d.get_height()!=n:
			d.set_size_request(w,n)
			r_offset.cnged(win.get_vadjustment(),h)
			return True
	return False

def forward(a,b):
	n=draw.offset+draw.size
	if n<draw.length:
		draw.offset+=a
		r_offset.cged(b)
		draw.redraw()
def backward(a,b):
	if draw.offset>0:
		draw.offset-=a
		r_offset.cged(b)
		draw.redraw()
#def edge(wn,pos,d):
#	if pos==Gtk.PositionType.RIGHT:
#		forward(size,win.get_hadjustment())
#	elif pos==Gtk.PositionType.BOTTOM:
#		forward(size,win.get_vadjustment())
#	elif pos==Gtk.PositionType.LEFT:
#		backward(min(size,draw.offset),win.get_hadjustment())
#	else:
#		backward(min(size,draw.offset),win.get_vadjustment())
def edge(val,max):
	if landscape:
		ad=win.get_hadjustment()
		if val==0:
			backward(min(size,draw.offset),ad)
		elif val==max:
			forward(size,ad)
	else:
		ad=win.get_vadjustment()
		if val==0:
			backward(min(size,draw.offset),ad)
		elif val==max:
			forward(size,ad)

def init():
	global win
	win=Gtk.ScrolledWindow(vexpand=True)
	win.set_child(draw.init())
	#win.connect('edge-overshot',edge,None)
	win.get_hadjustment().connect('value-changed',r_offset.cgd,None)
	win.get_vadjustment().connect('value-changed',r_offset.cgd,None)

def move(b,next):
	if landscape:
		a=int(win.get_width()/2)
		b=win.get_hadjustment()
	else:
		a=int(win.get_height()/2)
		b=win.get_vadjustment()
	if next:
		forward(a,b)
	else:
		backward(a,b)

def open():
	global control,control2
	control=Gtk.EventControllerKey()
	control.connect("key-pressed",eve,None)
	win.add_controller(control)
	control2 = Gtk.GestureClick()
	control2.connect("pressed",press,win)
	win.add_controller(control2)
def close():
	win.remove_controller(control)
	win.remove_controller(control2)
def eve(controller,keyval,keycode,state,d):
	if keyval==Gdk.KEY_period: # or keyval==Gdk.KEY_greater
		seloff.moveright.emit(sets._click_)
		return True
	elif keyval==Gdk.KEY_comma: # or keyval==Gdk.KEY_less
		seloff.moveleft.emit(sets._click_)
		return True
	elif keyval==Gdk.KEY_Left and point.lastselect:
		step.leftk()
		return True
	elif keyval==Gdk.KEY_Right and point.lastselect:
		step.rightk()
		return True
	elif keyval==Gdk.KEY_Up and point.lastselect:
		step.upk()
		return True
	elif keyval==Gdk.KEY_Down and point.lastselect:
		step.downk()
		return True
	return False
def press(g,n,x,y,w):
	w.grab_focus() #it's tested at this call




from gi.repository import Gtk

from . import sets
from . import draw
from . import points
from . import save
from . import move
from . import point
from . import distance

dif=Gtk.EntryBuffer()

#signbutton,maxlabel
sign_positive="+"

#box,distancebutton,pointsorig,pointsorigh,samplesorig

def open(b,combo):
	global signbutton,maxlabel,calculated,middlerate,pausesbool,anchorbool
	global box,distancebutton,pointsorig,pointsorigh,samplesorig #since pauses can be more points
	box=Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
	#+/- button or not   entry   maxim
	b2=Gtk.Box()
	if draw.baseline!=0:
		signbutton=sets.colorButton(sign_positive,sign,"Sign")
		b2.append(signbutton)
	en=sets.colorEntry(dif)
	b2.append(en)
	b2.append(sets.colorLabel("from 0 to "))
	maxlabel=sets.colorLabel(maximum())
	b2.append(maxlabel)
	box.append(b2)
	#atstart   middle[-1,1]   - or calculated
	cdata,mdata=calculate()
	st=sets.colorLabel(cdata)
	middlerate=sets.colorLabel(mdata)
	calculated=sets.colorLabel("-")
	b3=Gtk.Box(homogeneous=True)
	b3.append(st)
	b3.append(middlerate)
	b3.append(calculated)
	box.append(b3)
	b4=Gtk.Grid()
	b4.attach(left_label("Keep pauses"),0,0,1,1)
	pausesbool=Gtk.CheckButton(active=True)
	b4.attach(pausesbool,1,0,1,1)
	b4.attach(left_label("Keep anchor points"),0,1,1,1)
	anchorbool=Gtk.CheckButton(active=True)
	b4.attach(anchorbool,1,1,1,1)
	box.append(b4)
	#Calculate
	calc=sets.colorButton("Set",calcs,"Calculate")
	box.append(calc)
	#Cancel
	exit=sets.colorButton("Cancel",abort,"Abort",combo)
	box.append(exit)
	bt=sets.colorButton("Return",click,"Finish",combo)
	box.append(bt)
	bt=sets.colorButton("Done",ready,"Set & Return",combo)
	box.append(bt)

	#copies
	#.copy() => it is not deep, _height_ same
	pointsorig=points.points.copy()
	pointsorigh=[]
	for p in points.points:
		pointsorigh.append(p._height_)

	samplesorig=draw.samples.copy()
	distancebutton=None
	#and set
	combo[0].set_child(box)
def left_label(a):
	b=sets.colorLabel(a)
	b.set_halign(Gtk.Align.START)
	return b

def click(b,combo):
	finish(combo)
def finish(combo):
	if not (dtest:=distance.test_all()):
		conclude(combo)
	else:
		global distancebutton
		a=distancebutton
		distancebutton=distance.hold(dtest,distancebutton,callback,combo)
		if not a:
			box.append(distancebutton)
def conclude(combo):
	move.saved(combo) #this here, else problems at get_native().get_surface()
	if sets.get_fulleffect():
		save.saved()
	else:
		abort_samples()
def callback(b,combo):
	conclude(combo)

def sign(b,d):
	if b.get_child().get_text()==sign_positive:
		b._set_text_("-")
	else:
		b._set_text_(sign_positive)
	maxlabel._set_text_(maximum())

def abort_samples():
	draw.samples=samplesorig
def abort(b,combo):
	restore()
	abort_samples()
	done(combo)
def restore():
	points.points.clear()
	for i in range(0,len(pointsorigh)):
		points.points.append(pointsorig[i])
		points.points[i]._height_=pointsorigh[i]

def size_sign():
	if draw.baseline!=0:
		positiv=signbutton.get_child().get_text()==sign_positive
	else:
		positiv=True
	return (get_size(),positiv)
def get_size():
	if draw.baseline!=0:
		return int(draw.sampsize*draw.baseline)
	return draw.sampsize

def maximum():
	a,positiv=size_sign()
	a-=1 #not targeting 32768, but [0,32767]
	x=0
	for p in points.points:
		if p._height_>=0:
			h=p._height_
		else:
			h=-p._height_
		if positiv:
			h=a-h
		if h>x:
			x=h
	return x.__str__()

def not_a_digit(buf):
	buf.set_text("0",-1) #isdigit failed

def psign(r,p,a,sz):
	if r._height_>=0:
		p._height_+=a
		if p._height_>=sz:
			p._height_=sz-1
	else:
		p._height_-=a
		if p._height_<-sz:
			p._height_=-sz
def calcs(b,d):
	modify()
def modify():
	c=dif.get_text()
	if c.isdigit():
		a=int(c)
		b=int(maxlabel.get_text())
		if a>b:
			dif.set_text(b.__str__(),-1)
			return False
		restore() #need no more points or points tend to flat
		sz,sgn=size_sign()
		pauses=[]
		if sgn:
			rng=len(points.points)
			for i in range(0,rng):
				if pause(i,pauses) and anchor(i):
					p=points.points[i]
					if p._height_==0 and rng>1:
						#at positiv loud need to go like sibling if it's 0 not plus only
						if i>0:
							psign(points.points[i-1],p,a,sz)
						else:
							psign(points.points[i+1],p,a,sz)
					else:
						psign(p,p,a,sz)
		else:
			rng=len(points.points)
			for i in range(0,rng):
				p=points.points[i]
				if p._height_>=0:
					if a>=p._height_:
						p._height_=0
					else:
						p._height_-=a
				else:
					if a>=-p._height_:
						p._height_=0
					else:
						p._height_+=a
		resolve(pauses)
		maxlabel._set_text_(maximum())
		save.apply()
		cdata,mdata=calculate()
		calculated._set_text_(cdata)
		middlerate._set_text_(mdata)
		return True
	not_a_digit(dif)
	return False

def done(combo):
	combo[0].set_child(combo[1])

def ready(b,combo):
	if modify():
		finish(combo)

def calculate():
	s=len(points.points)
	if s>=2:
		n=0
		start=points.points[0]._offset_
		stop=points.points[s-1]._offset_
		if stop>draw.length: #here and 2 more places (about samples length vs points length)
			stop=draw.length
		for i in range(start,stop):
			n+=abs(draw.samples[i])
		med=n/(stop-start)
		#
		a=get_size()/2
		b=med-a
		mid=b/a
		#
		return (med.__str__(),mid.__str__())
	return ("-","-")

def pause(i,lst):
	if pausesbool.get_active():
		a=points.points[i]
		if a._height_==0:
			j=i+1
			first=len(lst)==0
			if j!=len(points.points):
				b=points.points[j]
				if b._height_==0:
					#here is a sound pause
					if first or lst[len(lst)-1]!=i:
						#is new
						lst.append(i)
						lst.append(j)
						if first:
							#the interval start if it is at the start it will not be adjusted
							return False
					else:
						#extend only
						lst[len(lst)-1]=j
						return False
			elif (not first) and lst[len(lst)-1]==i:
				#last that is in pause needs false
				return False
	return True
def anchor(i):
	if anchorbool.get_active():
		if i>0 and i+1<len(points.points) and points.points[i]._height_==0 and points.points[i-1]._height_!=0 and \
		points.points[i+1]._height_!=0 and ((points.points[i-1]._height_^points.points[i+1]._height_)<0):
			return False
	return True

def resolve(pauses):
	sz=len(pauses)
	of=0
	i=0
	while i<sz: #for-loop will not care about i modification inside
		a=pauses[i]
		if a>0:
			insert(a+of,1)
			of+=1
		i+=1
		a=pauses[i]
		if a+1<len(points.points):
			insert(a+of,0)
			of+=1
		i+=1

def insert(ix,more):
	p=points.points[ix]
	points.add(p._offset_,0,False,point.default_concav,ix+more)




from gi.repository import Gtk

from . import info
from . import sets

def open(e):
	box=Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
	box.append(sets.colorLabel("The following exception have been occurred:"))
	box.append(sets.colorLabel(e))
	box.append(sets.colorButton("OK",done,"Acknowledgement"))
	info.win.set_child(box)

def done(b,d):
	info.win.set_child(info.box)




from . import drawscroll
from . import graph
from . import point

import math

def draw(cr,x0,y0,x1,y1,concav):
	land=drawscroll.landscape
	if land:
		x=x1-x0
		y=abs(y1-y0)
	else:
		x=abs(x1-x0)
		y=y1-y0
	c,raddif=radius(x,y)
	xc,yc,rstart,rend=center(x0,y0,x1,y1,concav,x,y,c,raddif,land)
	#will be a heavy cpu load with radsmall when moveing an inter up a stable at angle 0 cr.arc(xc,yc,c,rstart+radsmall,rend-radsmall)
	cr.arc(xc,yc,c,rstart,rend)
def center(x0,y0,x1,y1,concav,x,y,c,raddif,land=True):
	if land:
		if concav: #concav on land
			if x>y:
				if y0<y1:
					#...
					#   ...
					xc=x0
					yc=y0+c
					#if raddif!=None:
					a=math.pi*3/2
					b=math.pi*3/2+raddif
				else:
					#   ...
					#...
					xc=x1
					yc=y1+c
					a=math.pi*3/2-raddif
					b=math.pi*3/2
			else:
				if y0<y1:
					#.
					#.
					# .
					# .
					xc=x1-c
					yc=y1
					a=-raddif
					b=0
				else:
					# .
					# .
					#.
					#.
					xc=x0+c
					yc=y0
					a=math.pi
					b=math.pi+raddif
		else:
			if x>y:
				if y0<y1:
					#...
					#   ...
					xc=x1
					yc=y1-c
					a=math.pi/2
					b=math.pi/2+raddif
				else:
					#   ...
					#...
					xc=x0
					yc=y0-c
					a=math.pi/2-raddif
					b=math.pi/2
			else:
				if y0<y1:
					#.
					#.
					# .
					# .
					xc=x0+c
					yc=y0
					a=math.pi-raddif
					b=math.pi
				else:
					# .
					# .
					#.
					#.
					xc=x1-c
					yc=y1
					a=0
					b=raddif
	else:
		if concav:
			if x>y:
				if x0<x1:
					#...
					#   ...
					xc=x1
					yc=y1-c
					a=math.pi/2
					b=math.pi/2+raddif
				else:
					#   ...
					#...
					xc=x0
					yc=y0+c
					a=math.pi*3/2-raddif
					b=math.pi*3/2
			else:
				if x0<x1:
					#.
					#.
					# .
					# .
					xc=x0+c
					yc=y0
					a=math.pi-raddif
					b=math.pi
				else:
					# .
					# .
					#.
					#.
					xc=x1+c
					yc=y1
					a=math.pi
					b=math.pi+raddif
		else:
			if x>y:
				if x0<x1:
					#...
					#   ...
					xc=x0
					yc=y0+c
					a=math.pi*3/2
					b=math.pi*3/2+raddif
				else:
					#   ...
					#...
					xc=x1
					yc=y1-c
					a=math.pi/2-raddif
					b=math.pi/2
			else:
				if x0<x1:
					#.
					#.
					# .
					# .
					xc=x1-c
					yc=y1
					a=-raddif
					b=0
				else:
					# .
					# .
					#.
					#.
					xc=x0-c
					yc=y0
					a=0
					b=raddif
	return (xc,yc,a,b)
def radius(x,y):
	if x>y:
		n=x
		m=y
	else:
		n=y
		m=x
	rbig=graph.rads(n,m)
	rsmall=math.pi/2-rbig
	dif=rbig-rsmall
	cat2=n/math.cos(dif)
	return (cat2,math.pi/2-dif)
#def vals(x,y):
#	c,raddif,rads=radius(x,y)
#	l=point.const
#	aux=math.cos(rads)
#	ad=aux*l
#	op=math.sin(rads)*l
#	radsmall=math.atan2(op,c-ad)
#	return (c,raddif,radsmall)




from gi.repository import Gtk

from . import play
from . import sets

#win,box

def open(b,d):
	bx=Gtk.Grid(hexpand=True)
	nchannels, sampwidth, framerate, nframes, comptype, compname = play.wavefile.getparams()
	add(bx,"Name","Value",0,'b')
	add(bx,"nchannels",nchannels.__str__(),1)
	add(bx,"sampwidth",sampwidth.__str__(),2)
	add(bx,"framerate",framerate.__str__(),3)
	add(bx,"nframes",nframes.__str__(),4)
	add(bx,"comptype",comptype,5)
	add(bx,"compname",compname,6)
	bx.attach(sets.colorButton("Done",done,"Back"),0,7,2,1)
	win.set_child(bx)

def add(b,n,v,r,e=None):
	a=sets.colorLabel(n,e)
	a.set_halign(Gtk.Align.START)
	f = Gtk.Frame()
	f.set_hexpand(True)
	f.set_child(a)
	b.attach(f,0,r,1,1)
	a=sets.colorLabel(v,e)
	a.set_halign(Gtk.Align.START)
	f = Gtk.Frame()
	f.set_hexpand(True)
	f.set_child(a)
	b.attach(f,1,r,1,1)

def done(b,d):
	win.set_child(box)




from gi.repository import Gtk

from . import sets
from . import points
from . import level
from . import draw
from . import blank
from . import move
from . import save
from . import distance

spread=Gtk.EntryBuffer()
reduce=Gtk.CheckButton()

#box,pointsorig,samplesorig,distancebutton

def open(b,combo):
	global box
	box=Gtk.Grid(hexpand=True)
	box.attach(sets.colorLabel("Spread/Compress N samples"),0,0,1,1)
	box.attach(sets.colorEntry(spread),1,0,1,1)
	box.attach(sets.colorLabel("Compress"),0,1,1,1) #Enlarge
	box.attach(reduce,1,1,1,1)
	box.attach(sets.colorButton("Cancel",cancel,"Abort",combo),0,2,2,1)
	box.attach(sets.colorButton("Done",done,"Apply",combo),0,3,2,1)
	try:
		global pointsorig,samplesorig
		#if from previous compress
		del pointsorig
		del samplesorig
	except:
		pass
	global distancebutton
	distancebutton=None
	combo[0].set_child(box)

def cancel(b,combo):
	try:
		restore()
		draw.samples=samplesorig
	except:
		pass
	combo[0].set_child(combo[1])
def restore():
	for i in range(0,len(pointsorig)): #pointsorig can be modified it will not influence the starting for-to
		points.points[i]._offset_=pointsorig[i]

def done(b,combo):
	a=spread.get_text()
	if a.isdigit():
		ps=points.points
		s=len(ps)
		if s>=2:
			b=int(a)
			if reduce.get_active():
				try:
					#can be in vain , is return after, but who cares about that return
					restore()
				except:
					global pointsorig
					pointsorig=[]
					for p in points.points:
						pointsorig.append(p._offset_)
				end=ps[s-1]._offset_
				n=end-ps[0]._offset_
				if b>n:
					spread.set_text(n.__str__(),-1)
					return
				apply(-b,-1)
				if sets.get_fulleffect():
					global samplesorig
					try:
						draw.samples=samplesorig.copy() #can hit done multiple times and then is .copy
					except:
						samplesorig=draw.samples.copy()
					compress(b,end)
				if dtest:=distance.test_all():
					global distancebutton
					a=distancebutton
					distancebutton=distance.hold(dtest,distancebutton,callback,combo)
					if not a:
						box.attach(distancebutton,0,4,2,1)
					return
			else:
				if sets.get_fulleffect():
					enlarge(b)
				apply(b,1)
			conclude(combo)
			return
		combo[0].set_child(combo[1])
		return
	level.not_a_digit(spread)
def conclude(combo):
	move.saved(combo)
	if sets.get_fulleffect():
		blank.saved()
		save.effect()
def callback(b,combo):
	conclude(combo)

def enlarge(n):
	s=len(draw.samples)
	extra=s-points.points[len(points.points)-1]._offset_
	#there is a nicer move but this is lazy
	right=draw.samples[s-extra:]
	del draw.samples[s-extra:]
	draw.samples=draw.samples+([0]*n)+right

def compress(n,end):
	#copy right at position
	s=len(draw.samples)
	extra=s-end
	for i in range(end,s):
		draw.samples[i-n]=draw.samples[i]
	#remove N in safe
	del draw.samples[s-n:]

def apply(n,sign):
	ps=points.points
	leng=len(ps)
	total=ps[leng-1]._offset_-ps[0]._offset_
	rest=[]
	rest_sum=0
	for i in range(1,leng):
		sz=(ps[i]._offset_-ps[i-1]._offset_)*n
		sp=int(sz/total)
		rs=sz%total
		ps[i]._offset_+=sp
		rest.append([rs,i])
		rest_sum+=rs
		for j in range(i+1,leng):
			ps[j]._offset_+=sp
	rest.sort(reverse=True) #by first
	unassigned=int(rest_sum/total) #'float' object cannot be interpreted as an integer
	for i in range(0,unassigned):
		pos=rest[i][1]
		ps[pos]._offset_+=sign
		for j in range(pos+1,leng):
			ps[j]._offset_+=sign




from gi.repository import Gtk

from . import sets
from . import points
from . import draw
from . import graph
from . import point
from . import level
from . import save

buffer=Gtk.EntryBuffer()
to_right=Gtk.CheckButton()

def open(b,combo):
	bx=Gtk.Grid(hexpand=True)
	bx.attach(sets.colorLabel("Move points N samples"),0,0,1,1)
	bx.attach(sets.colorEntry(buffer),1,0,1,1)
	bx.attach(sets.colorLabel("Right direction"),0,1,1,1) #Left
	bx.attach(to_right,1,1,1,1)
	bx.attach(sets.colorButton("Cancel",cancel,"Abort",combo),0,2,2,1)
	bx.attach(sets.colorButton("Done",done,"Apply",combo),0,3,2,1)
	combo[0].set_child(bx)

def cancel(b,combo):
	combo[0].set_child(combo[1])

def saved(combo):
	combo[0].set_child(combo[1]) #here or problem at surface
	graph.redraw()
	if point.lastselect:
		point.lastselect._info_()

def done(b,combo):
	a=buffer.get_text()
	if a.isdigit():
		s=len(points.points)
		if s>=1:
			b=int(a)
			if to_right.get_active():
				dif=draw.length-points.points[s-1]._offset_
				if b>dif:
					buffer.set_text(dif.__str__(),-1)
					return
				b=-b
			else:
				start=points.points[0]._offset_
				if start<b:
					buffer.set_text(start.__str__(),-1)
					return
			for i in range(0,s):
				points.points[i]._offset_-=b
			saved(combo)
			if sets.get_fulleffect():
				save.effect()
			return
		combo[0].set_child(combo[1])
		return
	level.not_a_digit(buffer)




from . import points
from . import play
from . import draw
from . import save
from . import sets
from . import error

def init():
	global button
	button=sets.colorButton(chr(0x1F3D7),create,"Build")
	return button

def create(b,d):
	try:
		f=play.entry.get_text()
		#read points and build info
		info=points.read(f,True)
		if info:
			sampwidth,channels,rate,draw.length=info
			play.open(sampwidth,channels,rate)
			#samples
			draw.samples=[0] * draw.length # [0 for i in range(draw.length)]
			save.apply()
			#for playback
			play.save_file(f,sampwidth,channels,rate)
			play.waveopen(f)
	except Exception as e:
		error.open(e.__str__())




from . import seloff
from . import draw
from . import r_offset
from . import drawscroll
from . import reload

def act(b,d):
	st=seloff.start._get_()
	en=seloff.end._get_()
	del draw.samples[st:en]
	draw.length-=en-st
	rem=r_offset.atleft._get_()
	if(rem>=draw.length):
		#offset will be somewhere less than it is now
		n=drawscroll.win.get_width() if drawscroll.landscape else drawscroll.win.get_height()
		draw.offset=max(0,st-int(n/2))
	else:
		#reset
		draw.reset()
	draw.redraw()
	seloff.reset()
	changed()
	#for playback
	reload.file()

def changed():
	r_offset.cged(drawscroll.win.get_hadjustment()) if drawscroll.landscape else r_offset.cged(drawscroll.win.get_vadjustment())




from . import point
from . import points
from . import draw
from . import drawscroll
from . import snap

#tests: 1 right circle,2 right inter,3 right margin
#       4 left free/inter/margin
#       5 landscape circle (inter easiest)

def pos_type(ix):
	return points.points[ix]._offset_-draw.offset-point.const
def left_type():
	p=point.lastselect
	ix=points.points.index(p)
	if ix==0 or points.points[ix-1]._offset_<draw.offset:
		return -point.const
	return pos_type(ix-1)
def right_type():
	p=point.lastselect
	ix=points.points.index(p)
	if ix+1==len(points.points) or points.points[ix+1]._offset_>(draw.offset+draw.size):
		return draw.size-point.const
	return pos_type(ix+1)

def autodrag(p,x,y):
	snap.autodrag(p,x+point.const,y+point.const)

def left(b,d):
	leftk()
def leftk():
	p=point.lastselect
	if p.get_parent(): #only if in current cont, else will be an error, visible and gtk error
		if drawscroll.landscape:
			limit=left_type()
		else:
			limit=-point.const
		x,y=draw.cont.get_child_position(p)
		x-=1 #on height is double
		if x>limit:
			autodrag(p,x,y)

def right(b,d):
	rightk()
def rightk():
	p=point.lastselect
	if p.get_parent():
		if drawscroll.landscape:
			limit=right_type()
		else:
			limit=draw.area.get_width()-point.const
		x,y=draw.cont.get_child_position(p)
		x+=1
		if x<limit:
			autodrag(p,x,y)

def up(b,d):
	upk()
def upk():
	p=point.lastselect
	if p.get_parent():
		if drawscroll.landscape:
			limit=-point.const
		else:
			limit=left_type()
		x,y=draw.cont.get_child_position(p)
		y-=1
		if y>limit:
			autodrag(p,x,y)

def down(b,d):
	downk()
def downk():
	p=point.lastselect
	if p.get_parent():
		if drawscroll.landscape:
			limit=draw.area.get_height()-point.const
		else:
			limit=right_type()
		x,y=draw.cont.get_child_position(p)
		y+=1
		if y<limit:
			autodrag(p,x,y)




from gi.repository import Gtk

import os
import pathlib
import json
import time

from . import sets   #colorLabel,...
from . import play   #wavefile,entry
from . import draw   #samples,...
from . import points #points,...
from . import save   #apply,...
from . import point  #lastselect
from . import move   #saved
from . import pbox   #close

default_toler="1"
toler=Gtk.EntryBuffer(text=default_toler)

#there is a min dist in Settigs, why another one? That is for manually placed points.
default_mdist="5"
mdist=Gtk.EntryBuffer(text=default_mdist)

stop=Gtk.CheckButton()
default_stop="100"
stop_after=Gtk.EntryBuffer(text=default_stop)

print_test=Gtk.CheckButton()
tests_str1="dif sum "
tests_str2="points len "
tests_str3="phase1/orig "

skip_phase2=Gtk.CheckButton()
pause=Gtk.CheckButton()
default_pause="100"
pause_points=Gtk.EntryBuffer(text=default_pause)
pause_points_minimum=1

#tests_time

def data(b,combo):
	box=Gtk.Grid(hexpand=True)

	box.attach(sets.colorLabel("Tolerance"),0,0,2,1)
	box.attach(sets.colorEntry(toler),2,0,1,1)
	box.attach(sets.colorLabel("‰"),3,0,1,1)

	box.attach(sets.colorLabel("Min distance"),0,1,2,1)
	box.attach(sets.colorEntry(mdist),2,1,2,1)

	box.attach(sets.colorLabel("Stop after N non-inter points"),0,2,1,1)
	box.attach(stop,1,2,1,1)
	box.attach(sets.colorEntry(stop_after),2,2,2,1)

	box.attach(sets.colorLabel("Skip phase 2"),0,3,2,1)
	box.attach(skip_phase2,2,3,2,1)

	common_options(box,4)

	pos=6
	if fastpath(False):
		box.attach(sets.colorButton("Fast Resume",restore,"Restore",combo),0,pos,4,1)
		pos+=1

	box.attach(sets.colorButton("Cancel",cancel,"Abort",combo),0,pos,4,1)
	box.attach(sets.colorButton("Done",done,"Apply",combo),0,pos+1,4,1)
	combo[0].set_child(box)

def common_options(box,column):
	box.attach(sets.colorLabel("Verbosity"),0,column,2,1)
	box.attach(print_test,2,column,2,1)
	column+=1

	box.attach(sets.colorLabel("At phase 2, pause every N points"),0,column,1,1)
	box.attach(pause,1,column,1,1)
	box.attach(sets.colorEntry(pause_points),2,column,2,1)

def cancel(b,combo):
	combo[0].set_child(combo[1])

def done(b,combo):
	a=toler.get_text()
	abool=a.isdigit()
	b=mdist.get_text()
	bbool=b.isdigit()
	c=stop_after.get_text()
	cbool=c.isdigit()
	d=pause_points.get_text()
	dbool=d.isdigit()
	if abool and bbool and cbool and dbool:
		a=int(a)
		b=int(b)
		c=int(c)
		d=int(d)
		if a>1000:
			toler.set_text("1000",-1)
		elif b==0:
			mdist.set_text("1",-1)
		elif c<2:
			stop_after.set_text("2",-1)
		elif d<pause_points_minimum:
			pause_points.set_text(str(pause_points_minimum),-1)
		else:
			a=round(pow(2,8*play.wavefile.getsampwidth())*a/1000)

			points.points.clear()

			pack=worker(a,b,c,d,draw.samples.copy(),combo)
			if pack!=None:
				waiter(combo,pack)
	else:
		if not abool:
			toler.set_text(default_toler,-1)
		if not bbool:
			mdist.set_text(default_mdist,-1)
		if not cbool:
			stop_after.set_text(default_stop,-1)
		if not dbool:
			pause_points.set_text(default_pause,-1)

def precalculate1():
	points.add(0,0,False,True,0) #p1
	points.add(0,0,False,True,1) #p2
def precalculate2():
	points.add(0,0,False,True,2) #p3
	points.points[1]._inter_=True

#None/continue_pack
def calculate(samples,length,tolerance,min_dist,max,pause_after,samplesorig):
	#exclude blank extremes
	for i in range(0,length): #not including length element
		if samples[i]!=0:
			break
	for j in range(length-1,-1,-1):
		if samples[j]!=0:
			break
	j=j+1

	if (i+min_dist+1)<j: #only if there is a length of min 2 points
		pnts=[]
		pnts.append(points.newp(i,samples[i],False,True))

		precalculate1()

		if print_test.get_active():
			tests=0
			tests2=0
			tests3=0

		while (i+min_dist+1)<j:  #j can be length
			points.points[0]._offset_=i
			points.points[0]._height_=samplesorig[i]

			for k in range(i+min_dist+1,j):
				points.points[1]._offset_=k
				points.points[1]._height_=samplesorig[k]
				save.apply() #or save.apply_line, will exclude at right
				newdif=0
				for l in range(i,k):
					newdif+=abs(samples[l]-samplesorig[l])

				if newdif>tolerance: #get back one place
					k=k-1
					samples[k]=samplesorig[k]  #this can be restored at last, why not there? who cares, too much code
					points.points[1]._offset_=k
					points.points[1]._height_=samplesorig[k]
					save.apply()
					if print_test.get_active():
						newdif=0
						for l in range(i,k):
							newdif+=abs(samples[l]-samplesorig[l])
					break
			pnts.append(points.newp(k,samplesorig[k],False,True)) #'struct' object has no attribute 'copy'

			if print_test.get_active():
				tests+=newdif
				tests2+=k-i
				for l in range(i,k):
					tests3+=abs(samplesorig[l])

			if stop.get_active():
				if len(pnts)==max:
					break

			i=k
		if print_test.get_active():
			print(tests_str1+str(tests))  #the two tolerances at start will trade precision for more code
			print(tests_str2+str(len(pnts)))
			print(tests_str3+str(tests/tests3))
			print("avg dist "+str(tests2/len(pnts)))
			testspack=[0,0,0,0,0]
		else:
			testspack=[]

		#phase 2 apply arcs for better match
		pnts2=pnts.copy()
		if skip_phase2.get_active()==False:
			precalculate2()

			i,ix,is_done=calculate_resume(pnts,pnts2,samples,samplesorig,0,0,pause_after,testspack)
			if is_done==False:
				return [pnts,pnts2,samples,samplesorig,i,ix,pause_after,testspack]

		points.points=pnts2
	return None

def tests_phase2(pnts2,testspack,remainings):
	print()
	print(tests_str1+str(testspack[0]))
	print(tests_str2+str(len(pnts2)))
	p1_o=testspack[2]/testspack[1]
	print(tests_str3+str(p1_o))
	d_o=testspack[0]/testspack[1]
	print("dif/orig "+str(d_o))
	print(" change "+str(d_o-p1_o))

	t=time.time()-tests_time
	testspack[3]+=1
	testspack[4]+=t
	print("seconds/pauses "+str(int(testspack[4]/testspack[3]))) #time.time() is float
	if remainings:
		print("remaining points "+str(remainings))

def arc(a,b,xleft,xright,ystart,yend,bestmatch,samples,samplesorig):
	points.points[0]._concav_=a;points.points[1]._concav_=b
	for x in range(xleft,xright):
		points.points[1]._offset_=x
		for y in range(ystart,yend):
			points.points[1]._height_=y
			save.apply() #or save.apply_arc

			thisdif=0
			for k in range(xleft,xright):
				thisdif+=abs(samples[k]-samplesorig[k])

			if thisdif<bestmatch[0]:
				bestmatch[0]=thisdif
				bestmatch[1]=a
				bestmatch[2]=b
				bestmatch[3]=x
				bestmatch[4]=y

def waiter(combo,pack):
	box=Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
	box.append(sets.colorLabel("Resume Screen"))
	box.append(sets.colorButton("Fast Resume Save &amp; Done",fastsavedone,"Restore Save & Stop",[pack,combo]))
	box.append(sets.colorButton("Fast Resume Save &amp; Resume",fastsaveresume,"Restore Save & Resume",[pack,combo]))
	box.append(sets.colorButton("Resume",resumefn,"Continue",[pack,combo]))
	combo[0].set_child(box)

def resumefn(b,bigpack):
	pack,combo=bigpack
	resume(combo,pack)

def terminate(combo,pnts2,samplesorig,testspack):
	points.points=pnts2
	finish(samplesorig,combo)

def resume(combo,pack):
	pnts2=pack[1];samplesorig=pack[3];testspack=pack[7]
	pack[4],pack[5],is_done=calculate_resume(pack[0],pnts2,pack[2],samplesorig,pack[4],pack[5],pack[6],testspack)
	if is_done:
		terminate(combo,pnts2,samplesorig,testspack)
		fastsavefinish()

#same as calculate+for finish
def worker(tolerance,min_dist,max,pause_after,samplesorig,combo): #used to not store height on another var at phase1, and at phase2
	pack=calculate(draw.samples,draw.length,tolerance,min_dist,max,pause_after,samplesorig)
	if pack!=None:
		return pack
	finish(samplesorig,combo)
	fastsavefinish()
	return None

def finish(samplesorig,combo):
	if point.lastselect:
		pbox.close()
		point.lastselect=None
	if not sets.get_fulleffect():
		draw.samples=samplesorig
	move.saved(combo)
	if sets.get_fulleffect():
		save.saved()

#i,ix,is_done
def calculate_resume(pnts,pnts2,samples,samplesorig,i,ix,pause_after,testspack):
	if print_test.get_active():
		global tests_time
		tests_time=time.time()

	aux=points.points[1]
	sz=len(pnts)-1
	for i in range(i,sz):
		xleft=pnts[i]._offset_;xright=pnts[i+1]._offset_
		ystart=pnts[i]._height_;yend=pnts[i+1]._height_
		points.points[0]._offset_=xleft;points.points[0]._height_=ystart
		points.points[2]._offset_=xright;points.points[2]._height_=yend

		#calculate current dif
		startdif=0
		for k in range(xleft,xright):
			startdif+=abs(samples[k]-samplesorig[k])

		bestmatch=[startdif]+([None]*4)
		arc(True,True  ,xleft,xright,ystart,yend,bestmatch,samples,samplesorig)
		arc(True,False ,xleft,xright,ystart,yend,bestmatch,samples,samplesorig)
		arc(False,True ,xleft,xright,ystart,yend,bestmatch,samples,samplesorig)
		arc(False,False,xleft,xright,ystart,yend,bestmatch,samples,samplesorig)

		if bestmatch[0]!=startdif:
			points.points[0]._concav_=bestmatch[1];points.points[1]._concav_=bestmatch[2]
			points.points[1]._offset_=bestmatch[3];points.points[1]._height_=bestmatch[4]
			save.apply()

			pnts2[ix]._concav_=points.points[0]._concav_
			ix+=1
			pnts2.insert(ix,points.newp(points.points[1]._offset_,points.points[1]._height_,True,points.points[1]._concav_))
		else: #restore the line
			points.points[1]=points.points[2]
			points.points.pop()
			save.apply()
			points.points.insert(1,aux)
		ix+=1

		if print_test.get_active():
			testspack[0]+=bestmatch[0]
			for l in range(xleft,xright):
				testspack[1]+=abs(samplesorig[l])
			print(" "+str(i+1),end='',flush=True)  #+1? at least on 50 will be 1,2,...,49
			testspack[2]+=startdif    #again, for changes

		if pause.get_active():
			if ((i+2)%pause_after)==0:  #example: stop after 3, pase after 2, will stop at 2
				if (i+1)<sz: #not if was last
					if print_test.get_active():
						tests_phase2(pnts2,testspack,len(pnts)-(i+2))
					return (i+1,ix,False)
	if print_test.get_active():
		tests_phase2(pnts2,testspack,0)
	return (i,ix,True)

def fastsave(pack):
	fp=fastpath(True)
	with open(fp,"w") as f:
		pk=[points.serialize(pack[0]),points.serialize(pack[1])]+pack[2:] #pack is needed outside
		json.dump(pk,f)

def fastpath(is_save):
	f_in=play.entry.get_text()
	p=points.dpath(f_in)
	f=points.fpath(f_in,"fastresume")
	if is_save:
		pathlib.Path(p).mkdir(exist_ok=True)
		return f
	return os.path.exists(f)

def restore(b,combo):
	with open(fastpath(True)) as f:
		if d:=f.read():
			pack=json.loads(d)

			box=Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
			box.append(sets.colorLabel("Fast Resume Continue"))

			print_test.get_parent().remove(print_test) #gtk critical without this
			pause.get_parent().remove(pause)
			bx=Gtk.Grid(hexpand=True)
			common_options(bx,0)
			box.append(bx)
			if not len(pack[7]):
				print_test.set_active(False)
				print_test.set_sensitive(False) #can't resume verbosity when was not
			else:
				print_test.set_active(True) #in case is not
			pause.set_active(True) #in case is not
			pause_points.set_text(str(pack[6]),-1)

			box.append(sets.colorButton("Continue",fastresumefn,"Start",[pack,combo]))
			combo[0].set_child(box)

def fastsavedone(b,bigpack):
	pack,combo=bigpack
	fastsave(pack)
	terminate(combo,pack[1],pack[3],pack[7])
def fastsaveresume(b,bigpack):
	pack,combo=bigpack
	fastsave(pack)
	resume(combo,pack)

def fastsavefinish():
	if fastpath(False):
		os.remove(fastpath(True))

def fastresumefn(b,bigpack):
	d=pause_points.get_text()
	dbool=d.isdigit()
	if dbool:
		d=int(d)
		if d<pause_points_minimum:
			pause_points.set_text(str(pause_points_minimum),-1)
		else:
			pack,combo=bigpack

			pack[6]=d
			if not print_test.get_sensitive():
				print_test.set_sensitive(True) #for another use at this
			elif not print_test.get_active():
				pack[7]=[] #user wants to unset verbosity from now

			points.points.clear()
			precalculate1()
			precalculate2()

			pack[0]=points.deserialize(pack[0])
			pack[1]=points.deserialize(pack[1])
			draw.samples=pack[2]

			waiter(combo,pack)
	else:
		pause_points.set_text(default_pause,-1)




from . import point
from . import points
from . import draw
from . import graph
from . import sets

import cairo
from gi.repository import Gdk

def open(p):
	global button
	button=sets.colorButton(set(p),conv_conc,"Arc Orientation")
	return button

def conv_conc(b,d):
	p=point.lastselect
	pnts=points.points
	ix=pnts.index(p)
	if pnts[ix]._inter_ or (ix<(len(pnts)-1) and pnts[ix+1]._inter_):
		if p.get_parent() or pnts[ix+1].get_parent():
			arc_change(p,pnts[ix+1])
			b._set_text_(set(p))
			return
	p._concav_=False if p._concav_ else True
	b._set_text_(set(p))

def set(p):
	if p._concav_:
		return chr(0x23dc)
	else:
		return chr(0x23dd)

def arc_change(p0,p1):
	w=draw.wstore
	h=draw.hstore
	cr=cairo.Context(graph.surface)
	ope=cr.get_operator()
	cr.set_operator(cairo.Operator.CLEAR)
	graph.clearline(cr,p0,p1,w,h)
	cr.set_operator(ope)
	#
	p0._concav_=False if p0._concav_ else True
	#
	co=Gdk.RGBA()
	if co.parse(sets.get_fgcolor2()):
		cr.set_source_rgb(co.red,co.green,co.blue)
	graph.line_draw(cr,p0,p1,w,h)
	graph.area.queue_draw()





from gi.repository import Gtk

from . import sets
from . import draw
from . import save
from . import delete
from . import level
from . import points

def saved():
	draw.length=len(draw.samples)
	delete.changed() #this is not required in all cases but when adding without modifying draw(draw is big enough), will be required

start=Gtk.EntryBuffer(text="0")
stop=Gtk.EntryBuffer(text="0")
zero=Gtk.CheckButton()

def open(b,combo):
	bx=Gtk.Grid(hexpand=True)
	bx.attach(sets.colorLabel("Blank samples at start"),0,0,1,1)
	bx.attach(sets.colorEntry(start),1,0,1,1)
	bx.attach(sets.colorLabel("Blank samples at end"),0,1,1,1)
	bx.attach(sets.colorEntry(stop),1,1,1,1)
	bx.attach(sets.colorLabel("Blank left of first point and right of last point"),0,2,1,1)
	bx.attach(zero,1,2,1,1)
	bx.attach(sets.colorButton("Cancel",cancel,"Abort",combo),0,3,2,1)
	bx.attach(sets.colorButton("Done",done,"Apply",combo),0,4,2,1)
	combo[0].set_child(bx)

def cancel(b,combo):
	combo[0].set_child(combo[1])

def done(b,combo):
	a=start.get_text()
	b=stop.get_text()
	abool=a.isdigit()
	bbool=b.isdigit()
	if abool or bbool:
		if zero.get_active():
			zero_outside()
		if abool:
			c=int(a)
			draw.samples=([0]*c)+draw.samples
		if bbool:
			c=int(b)
			draw.samples+=[0]*c
		combo[0].set_child(combo[1])
		saved()
		save.saved()
		return
	if not abool:
		level.not_a_digit(start)
	if not bbool:
		level.not_a_digit(stop)

def zero_outside():
	prev=points.points[0]._offset_
	cur=points.points[len(points.points)-1]._offset_
	for i in range(0,prev):
		draw.samples[i]=0
	for i in range(cur,draw.length):
		draw.samples[i]=0




import math

from . import points
from . import arc
from . import draw
from . import reload

def data(b,d):
	effect()
def effect():
	apply()
	saved()
def saved():
	reload.file() #for playback , there is a wavefile.readframes at the callback
	redraw()

def redraw():
	draw.surf()
	draw.reset()
	draw.area.queue_draw()

def set(i,v):
	try: #here and 2 more places (about samples length vs points length)
		draw.samples[i]=int(v) #there are troubles at write file without int
	except:
		pass #this can be ignored
def apply():
	s=len(points.points)
	if s>=2:
		for i in range(1,s):
			prev=points.points[i-1]
			cur=points.points[i]
			x0=prev._offset_
			y0=prev._height_
			x1=cur._offset_
			y1=cur._height_
			#print(str(i-1)+"="+str(x0)+","+str(i)+"="+str(x1))
			if cur._inter_ or prev._inter_:
				apply_arc(x0,y0,x1,y1,prev._concav_)
			else:
				apply_line(x0,y0,x1,y1)

def apply_arc(x0,y0,x1,y1,conc):
	x=x1-x0
	y=abs(y1-y0)
	#get radius
	radius,rads=arc.radius(x,y)
	#get center
	_,yc,rstart,rend=arc.center(x0,y0,x1,y1,conc,x,y,radius,rads)
	#iterate
	if rstart==0 or rend==math.pi or rstart==math.pi or rend==0:
	#x axis
		xpos=radius-x
		if rstart==0 or rend==0:
			startpoint=0
			stoppoint=x
			#print(4 if rend==0 else 1)
		else:
			startpoint=1
			stoppoint=x+1
			#print(2 if rstart==math.pi else 3)
		for i in range(startpoint,stoppoint):
			#supra radius
			a=xpos/radius
			#to radians
			a=math.acos(a)
			#height
			h=math.sin(a)*radius
			if rstart==0:
				set(x0+i,y1+h)
			elif rstart==math.pi:
				set(x1-i,y0-h)
			elif rend==math.pi:
				set(x1-i,y0+h)
			else:
			#rend==0
				set(x0+i,y1-h)
			xpos+=1
	else:
	#y axis
		if rstart==math.pi*3/2 or rend==math.pi/2:
			startpoint=x-1
			stoppoint=-1
			#print(6 if rstart==math.pi*3/2 else 7)
		else:
			startpoint=x
			stoppoint=0
			#print(5 if rstart==math.pi/2 else 8)
		for i in range(startpoint,stoppoint,-1):
			a=i/radius
			a=math.asin(a)
			h=math.cos(a)*radius
			if rstart==math.pi/2:
				set(x1-i,yc+h)
			elif rstart==math.pi*3/2:
				set(x0+i,yc-h)
			elif rend==math.pi/2:
				set(x0+i,yc+h)
			else:
			#rend==math.pi*3/2
				set(x1-i,yc-h)

def apply_line(x0,y0,x1,y1):
	if x0!=x1: #a will be inf and also no for
		x=x1-x0
		y=y1-y0
		a=y/x
		for i in range(0,x):
			y=a*i
			set(x0+i,y0+y)



import gi
gi.require_version("Gtk", "4.0")
from gi.repository import Gtk

from . import loop
from . import sets
from . import play
from . import drawscroll
from . import r_offset
from . import bar
from . import forms
from . import info

def main():
	sets.init()
	win = Gtk.Window()
	win.set_decorated(False)#such a heavy load here if True
	win.maximize()
	win.show()
	#while loop.n:
	play.init()
	drawscroll.init()
	box=Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
	combo=[win,box]
	box.append(bar.init(combo))
	box.append(drawscroll.win)
	box.append(forms.init(combo))
	box.append(r_offset.init())
	win.set_child(box)
	info.win=win
	info.box=box
	loop.main.run()

if __name__ == "__main__":
    main()




from gi.repository import Gtk

from . import sets
from . import draw
from . import point
from . import drawscroll
from . import pbox
from . import points
from . import graph
from . import level
from . import save
from . import spread
from . import blank
from . import move
from . import distance
from . import automation

#button
control=None
on=chr(0x25a1)
formal_write="Write Points"

def init(combo):
	b=Gtk.Box()#homogeneous=True
	global button,box
	box=Gtk.Box(halign=Gtk.Align.CENTER,hexpand=True) #Fill default
	button=sets.colorButton(on,toggle,"Points Mode")#halign CENTER
	box.append(button)
	bt=sets.colorButton(chr(0x1f4be),save.data,formal_write)#1f5ab
	box.append(bt)
	bt=sets.colorButton(chr(0x1F4E3),level.open,"Loudness",combo) #0x2021
	box.append(bt)
	bt=sets.colorButton(chr(0x1FA97),spread.open,"Spread",combo)
	box.append(bt)
	bt=sets.colorButton(chr(0x2194),move.open,"Move Points",combo)
	box.append(bt)
	bt=sets.colorButton(chr(0x2422),blank.open,"Blanks",combo)
	box.append(bt)
	bt=sets.colorButton(chr(0x1F3E7),automation.data,"Automation",combo)
	box.append(bt)
	for bt in box:
		bt.set_sensitive(False)
	b.append(box)
	return b

def open():
	for b in box:
		b.set_sensitive(True)
	point.lastselect=None

def close():
	if control:
		button.emit(sets._click_)
	for b in box:
		b.set_sensitive(False)
	sz=len(points.points)
	for i in range(sz-1,-1,-1):
		points.points[i]._remove_(i)
	if point.lastselect:
		pbox.close()

def clear():
	x=draw.cont.get_first_child()
	while x:
		y=x.get_next_sibling()
		draw.cont.remove(x)
		x=y
def redraw(w,h):
	clear()
	drawpoints(w,h)
def drawpoints(w,h):
	sz=len(points.points)
	for i in range (0,sz):
		if points.points[i]._offset_<draw.offset:
			continue
		for j in range(i,sz):
			p=points.points[j]
			if draw.offset+(w if drawscroll.landscape
				 else h)<p._offset_:
				if j>0:
					graph.put(j,p,w,h)
				return
			p._put_(w,h,j)
		return

def toggle(b,a):
	a=draw.cont
	global control
	if control:
		a.remove_controller(control)
		control=None
		b._set_text_(on)
	else:
		control = Gtk.GestureClick()
		control.connect("pressed",press,None)
		a.add_controller(control)
		b._set_text_(chr(0x25a0))

def press(g,n,x,y,d):
	if point.lastselect and point.lastselect._drag_==True:
		point.lastselect._dend_(x,y)
	else:
		if distance.test(x,y,None):
			point.struct(x,y)
	graph.area.queue_draw()




from gi.repository import Gtk

from . import sets
from . import seloff
from . import forms
from . import point
from . import points
from . import draw
from . import graph
from . import arcbutton
from . import step
from . import snap

def open(p):
	global box,info
	box=Gtk.Box()
	sep=Gtk.Separator() #margin-start=0
	sep.set_margin_start(4)
	sep.set_margin_end(4)
	box.append(sep)
	box.append(arcbutton.open(p))
	#box.append(sets.colorButton(chr(0x0077),manual,"Manual"))#0057
	#def manual(b,d):
	box.append(sets.colorButton(seloff.char_delete,delete,"Delete"))
	a=chr(0x2B05)
	box.append(sets.colorButton(a,step.left,"Left ("+a+")"))
	a=chr(0x27A1)
	box.append(sets.colorButton(a,step.right,"Right ("+a+")"))
	a=chr(0x2B06)
	box.append(sets.colorButton(a,step.up,"Up ("+a+")"))
	a=chr(0x2B07)
	box.append(sets.colorButton(a,step.down,"Down ("+a+")"))
	box.append(sets.colorButton(chr(0x2693),snap.base,"Snap to base"))
	box.append(sets.colorButton(chr(0x21E4),snap.left,"Snap Left"))
	box.append(sets.colorButton(chr(0x21E5),snap.right,"Snap Right"))
	box.append(sets.colorButton(chr(0x2912),snap.up,"Snap Up"))
	box.append(sets.colorButton(chr(0x2913),snap.down,"Snap Down"))
	info=sets.colorLabel(inf(p))
	info.set_hexpand(True) #Default value: FALSE
	info.set_halign(Gtk.Align.END)
	box.append(info)
	forms.box.set_halign(Gtk.Align.START)
	forms.box.set_hexpand(False)
	forms.box.get_parent().append(box)

def close():
	forms.box.get_parent().remove(box)
	forms.box.set_hexpand(True)
	forms.box.set_halign(Gtk.Align.CENTER)

def delete(b,d):
	p=point.lastselect
	ix=points.points.index(p)
	dels,puts=p._take_(ix)
	ix=and_inter(ix,dels,puts)
	graph.lines(dels,puts,draw.wstore,draw.hstore)
	p._remove_(ix)
	if p.get_parent():
		p.get_parent().remove(p)
	close()
	point.lastselect=None
	graph.area.queue_draw()
def and_inter(ix,dels,puts):
	pnts=points.points
	sz=len(pnts)
	if ix==0:
		if sz==1:
			return ix
		if and_inter_test(1):
			dels.append([dels[0][1],pnts[1]])
	elif ix==(sz-1):
		if and_inter_test(sz-2):
			dels.append([pnts[sz-3],dels[0][0]])
			ix-=1
	elif pnts[ix-1]._inter_:
		if and_inter_test(ix+1):
			aux=pnts[ix+1]
			dels.append([dels[1][1],aux])
			puts[0][1]=aux
	return ix
def and_inter_test(test):
	p=points.points[test]
	b=p._inter_
	if b:
		p._remove_(test)
		if p.get_parent():
			p.get_parent().remove(p)
	return b

def inf(p):
	return str(p._offset_)+' '+str(p._height_)




#if there is the problem like in info.md with python10:
# now: rpm modified pyaudio is at releases at my pyaudioo project. deb and appimage are ok
#
# for pypi and source:
#	apt download audapa
#	sudo dpkg --ignore-depends=python3-pyaudio -i audapa.......deb
#but then, to not see broken count now and then, must remove python3-pyaudio at audapa package dependencies from /var/lib/dpkg/status
#A SOLUTION: overwrite ./build/lib.linux-x86_64-3.10/_portaudio.cpython-310-x86_64-linux-gnu.so at python3-pyaudio equivalent

#pip install --user .
#pip uninstall audapa

pkname='audapa'

import pathlib
HERE = pathlib.Path(__file__).parent
README = (HERE / "info.md").read_text()
ver=(HERE / "v2").read_text()

#import subprocess
#import setuptools.command.build_py
#class BuildPyCommand(setuptools.command.build_py.build_py):
#  """Custom build command."""
#  def run(self):
#    subprocess.run(['touch','qwerty.so'])
#    subprocess.run(['mv','qwerty.so','audapa'])
#    #MANIFEST.in include audapa/qwerty.so
#    #and include_package_data=True
#    setuptools.command.build_py.build_py.run(self)

from setuptools import setup
setup(name=pkname,
	version=ver,
	packages=[pkname],
	#optionals
	#cmdclass={
	#	'build_py': BuildPyCommand
	#},
	#include_package_data=True,
	python_requires='>=3.8',
	install_requires=[
		"pycairo>=1.20.0","PyGObject>=3.40",
		"appdirs>=1.4.3",
		"PyAudio>=0.2.11"],
	description='Audio wave file manipulator',
	long_description=README,
	long_description_content_type="text/markdown",
	url='https://github.com/colin-i/audapa',
	author='colin-i',
	author_email='costin.botescu@gmail.com',
	license='MIT',
	entry_points = {
		'console_scripts': [pkname+'='+pkname+'.main:main']
	}
)




from gi.repository import Gtk
import json
import configparser

class colorLabel(Gtk.Label):
	def __init__(self,t,extratag=None):
		Gtk.Label.__init__(self)
		self._set_text_(t,extratag)
	def _set_text_(self,t,extratag=None):
		z="<span"#p is error
		if (c:=text_color.get_text()):
			z+=" color='"+c+"'"
		z+=">"
		if extratag: #can't set with Pango after.
			z+="<"+extratag+">"
		z+=t
		if extratag:
			z+="</"+extratag+">"
		z+="</span>"
		self.set_markup(z)
_click_ = "clicked"
class colorButton(Gtk.Button):
	def __init__(self,t,f,i,d=None):
		Gtk.Button.__init__(self,child=colorLabel(t))
		self.connect(_click_,f,d)
		self.set_tooltip_text(i)
	def _set_text_(self,t):
		self.get_child()._set_text_(t)
class colorEntry(Gtk.Entry):
	def __init__(self,b=Gtk.EntryBuffer()):
		Gtk.Entry.__init__(self,buffer=b,hexpand=True)
		self._color_()
	def _color_(self):
		if (c:=text_color.get_text()):
			cont=self.get_style_context()
			self._provider_=Gtk.CssProvider()
			self._provider_.load_from_data (b"entry { color: "+c.encode()+b"; }")
			cont.add_provider(self._provider_,Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
		else:
			self._provider_=None
	def _recolor_(self):
		if self._provider_:
			cont=self.get_style_context()
			cont.remove_provider(self._provider_)
		self._color_()

pkgname='audapa'
import appdirs
import os.path
import pathlib
from html import escape

from . import draw
from . import forms
from . import point

def get_config_dir():
	return pathlib.Path(appdirs.user_config_dir(pkgname))
def get_config_file():
	return os.path.join(get_config_dir(),'config.ini')

def get_data_dir():
	return pathlib.Path(appdirs.user_data_dir(pkgname))
def get_data_file(f):
	return os.path.join(get_data_dir(),f)

color=Gtk.EntryBuffer(text="purple")
def get_color():
	return color.get_text()
fgcolor=Gtk.EntryBuffer(text="red")
def get_fgcolor():
	return fgcolor.get_text()
fgcolor2=Gtk.EntryBuffer(text="green")
def get_fgcolor2():
	return fgcolor2.get_text()
fgcolor3=Gtk.EntryBuffer(text="blue")
def get_fgcolor3():
	return fgcolor3.get_text()
text_color=Gtk.EntryBuffer()
def get_text_color():
	return text_color.get_text()
turn_page=Gtk.CheckButton(active=True)
full_effect=Gtk.CheckButton(active=True)
def get_fulleffect():
	return full_effect.get_active()
distance=Gtk.EntryBuffer(text="10")
#(2*point.const).__str__() was good but will confilct with the example

def add(bx,tx,x,n):
	return adder(bx,tx,colorEntry(x),n)
def adder(bx,tx,x,n):
	t=colorLabel(tx)
	t.set_halign(Gtk.Align.START)
	bx.attach(t,0,n,1,1)
	bx.attach(x,1,n,1,1)
	return n+1
def sets(b,combo):
	bx=Gtk.Grid(hexpand=True)
	n=add(bx,"Stroke Color",color,0)
	n=add(bx,"Foreground Color",fgcolor,n)
	n=add(bx,"Foreground Color2",fgcolor2,n)
	n=add(bx,"Foreground Color3",fgcolor3,n)
	n=add(bx,"Text Color",text_color,n)
	n=adder(bx,"Turn the page at margin touch",turn_page,n)
	n=adder(bx,forms.formal_write+" after a points effect",full_effect,n)
	n=add(bx,"Minimum distance between points",distance,n)
	b=colorButton("Done", reset, "Return", {'c':combo,'t':
		{'cl':color.get_text(),'fcl':fgcolor.get_text()}})
	bx.attach(b,0,n,2,1)
	combo[0].set_child(bx)

def init():
	get_config_dir().mkdir(exist_ok=True)
	get_data_dir().mkdir(exist_ok=True)
	config = configparser.ConfigParser()
	if(config.read(get_config_file())):
		c=config['conf']
		init_t(c,'color',color)
		init_t(c,'fgcolor',fgcolor)
		init_t(c,'fgcolor2',fgcolor2)
		init_t(c,'fgcolor3',fgcolor3)
		init_t(c,'text_color',text_color)
		init_c(c,'turn',turn_page)
		init_c(c,'effect',full_effect)
		init_t(c,'distance',distance)
def init_t(src,key,dst):
	if key in src: #this is not checking values
		dst.set_text(src[key],-1)
def init_c(src,key,dst):
	if key in src:
		dst.set_active(False if src[key]=='False' else True)

def reset(b,di):
	config = configparser.ConfigParser()
	config['conf']={}
	c=config['conf']
	c['color']=color.get_text()
	c['fgcolor']=fgcolor.get_text()
	c['fgcolor2']=fgcolor2.get_text()
	c['fgcolor3']=fgcolor3.get_text()
	c['text_color']=text_color.get_text()
	c['turn']=turn_page.get_active().__str__()
	c['effect']=full_effect.get_active().__str__()
	c['distance']=distance.get_text()
	with open(get_config_file(), "w") as configfile:
		config.write(configfile)
	win=di['c'][0]
	box=di['c'][1]
	if di['t']['cl']==c['color']:
		win.set_child(box)
		if di['t']['fcl']!=c['fgcolor']:
			draw.draw_sel()
		return
	draw.reset()
	search(box)
	win.set_child(box)
def search(p):
	x=p.get_first_child()
	while x:
		if isinstance(x,colorLabel):
			x._set_text_(escape(x.get_text()))
		elif isinstance(x,colorEntry):
			x._recolor_()
		else:
			search(x)
		x=x.get_next_sibling()




from gi.repository import Gtk,Gdk,GLib

from . import sets
from . import draw
from . import drawscroll
from . import pbox
from . import points
from . import graph
from . import arcbutton
from . import distance

const=6 #this is the radius
default_concav=True

class struct(Gtk.DrawingArea):
	_drag_=False
	#offset,height,inter,concav
	#                    here is like the Arc Orientation button that at start is /\ that is concav
	#                    https://math.stackexchange.com/questions/2364116/how-to-remember-which-function-is-concave-and-which-one-is-convex
	#control
	def __init__(self,*args):
		Gtk.DrawingArea.__init__(self)
		self._control_ = Gtk.GestureClick()
		self._control_.connect("pressed",self._press_,None)
		self.add_controller(self._control_)
		self.set_size_request(2*const,2*const)
		if len(args)==0:
			self.set_draw_func(self._draw_none_,None,None)
			return
		self._inter_=False
		self._concav_=default_concav
		self._pos_(args[0],args[1])
		ix=points.insert(self)
		puts,dels=self._take_(ix)
		w=draw.wstore
		h=draw.hstore
		graph.lines(dels,puts,w,h)
		self._put_point_(w,h)
		self._control_.emit("pressed",0,0,0)
	def _take_(self,ix):
		if a:=graph.take(ix,self):
			if len(a)==2:
				return (a,[[a[0][0],a[1][1]]])
			return (a,[])
		return ([],[])
	def _pos_(self,x,y):
		if drawscroll.landscape:
			o=x
			h=draw.sampsize*y/draw.hstore
		else:
			o=y
			h=draw.sampsize*x/draw.wstore
		self._offset_=int(draw.offset+o)
		self._height_=int(h-(draw.sampsize*draw.baseline))
	def _color_(self):
		if self._inter_==False:
			return sets.get_fgcolor2()
		return sets.get_fgcolor3()
	def _draw_none_(self,widget,cr,width,height,d,u):
		co=Gdk.RGBA()
		if co.parse(self._color_()):
			cr.set_source_rgb(co.red,co.green,co.blue)
		if self._drag_:
			cr.arc(const,const,const,0,2*GLib.PI)
		else:
			cr.rectangle(0,0,width,height)
		cr.stroke()
	def _draw_cont_(self,widget,cr,width,height,d,u):
		co=Gdk.RGBA()
		if co.parse(self._color_()):
			cr.set_source_rgb(co.red,co.green,co.blue)
		if self._drag_:
			cr.arc(const,const,const,0,2*GLib.PI)
		else:
			cr.rectangle(0,0,width,height)
		cr.fill()
	def _put_(self,w,h,ix):
		graph.put(ix,self,w,h)
		c=self._coord_(w,h)
		draw.cont.put(self,c[0]-const,c[1]-const)
	def _put_point_(self,w,h):
		c=self._coord_(w,h)
		draw.cont.put(self,c[0]-const,c[1]-const)
	def _coord_(self,w,h):
		z=self._offset_-draw.offset
		if drawscroll.landscape:
			y=self._height_*h/draw.sampsize
			y+=h*draw.baseline
			return [z,y]
		y=self._height_*w/draw.sampsize
		y+=w*draw.baseline
		return [y,z]
	def _press_(self,a,n,x,y,d):
		global lastselect
		if lastselect:
			if lastselect!=self:
				lastselect.set_draw_func(lastselect._draw_none_,None,None)
				self._info_()
				arcbutton.button._set_text_(arcbutton.set(self))
			else:
				if self._drag_==False:
					self._drag_=True
				else:
					self._drag_=False
				self.queue_draw()
				return
		else:
			pbox.open(self)
		lastselect=self
		self.set_draw_func(self._draw_cont_,None,None)
	def _dend_(self,x,y):
		if distance.test(x,y,self):
			w=draw.wstore
			h=draw.hstore
			#
			ini=points.points.index(self)
			if dels:=graph.take(ini,self):
				graph.dels(dels,w,h)
			#
			o=self._offset_
			self._pos_(x,y)
			if puts:=points.move(self,o,ini,dels):
				graph.lines(dels,puts,w,h)
			if self.get_parent():
				c=self._coord_(w,h)
				draw.cont.move(self,c[0]-const,c[1]-const)
			else:
				self._put_point_(w,h)
			self._info_()
	def _remove_(self,ix):
		self.remove_controller(self._control_)
		del points.points[ix]
	def _info_(self):
		pbox.info._set_text_(pbox.inf(self))




import math

import decimal
decimal.getcontext().prec=1
decimal.getcontext().rounding = decimal.ROUND_DOWN

from . import draw
from . import sets
from . import point
from . import points
from . import pbox

def test(x,y,p):
	d=float(sets.distance.get_text())
	if d:
		a=draw.cont.get_last_child() #iterate from last to first
		return recurse(x,y,p,a,d)
	return True

def check(a,b,d):
	x=a[0]+point.const-b[0]
	y=a[1]+point.const-b[1]
	c=math.sqrt(pow(x,2)+pow(y,2))
	if c<d: #<= can be another way
		e=decimal.Decimal(c%1)
		return int(c)+e.normalize().__float__()
	return -1

def recurse(x,y,p,a,d):
	if a:
		if a!=p:
			c=check(draw.cont.get_child_position(a),[x,y],d)
			if c!=-1:
				print(c.__str__()+" is less than the minimum required distance of "+d.__str__())
				return False
		return recurse(x,y,p,a.get_prev_sibling(),d)
	return True

def test_all():
	sz=len(points.points)
	if sz:
		d=float(sets.distance.get_text())
		a=points.points[0]._coord_(draw.wstore,draw.hstore)
		for i in range(1,sz):
			p=points.points[i]
			b=p._coord_(draw.wstore,draw.hstore)
			c=check(a,b,d)
			if c!=-1:
				return (p,c)
			a=b
	return None

def hold(p,b,f,d):
	if not b:
		b=sets.colorButton(text(p),f,"Confirm",d)
		return b
	b._set_text_(text(p))
	return b
def text(z):
	return "Distance problem at "+pbox.inf(z[0])+": "+z[1].__str__()




import os
import pathlib
import json

from . import sets
from . import point
from . import graph
from . import play
from . import draw

points=[]

def insert(poi):
	ln=len(points)
	for ix in range(0,ln):
		p=points[ix]
		if poi._offset_<p._offset_:
			points.insert(ix,poi)
			if ix>0:
				if (points[ix-1]._inter_==False
					 and p._inter_==False):
					poi._inter_=True
			return ix
	points.append(poi)
	return ln
def move(p,o,ini,dels):
	ix=ini
	last=len(points)-1
	of=p._offset_
	forward=o<of
	if forward:
		while ix<last:
			o=points[ix+1]._offset_
			if o<of:
				if p._inter_:
					p._offset_=o
					dels.clear()
					return graph.take(ix,p)
				ix+=1
				continue
			break
	else:
		while ix>0:
			o=points[ix-1]._offset_
			if of<o:
				if p._inter_:
					p._offset_=o
					dels.clear()
					return graph.take(ix,p)
				ix-=1
				continue
			break
	if ini!=ix:
		return move_inter(forward,ini,ix,dels,p)
	if dels:
		dels.clear()
	return graph.take(ix,p)
def move_inter(forward,ini,ix,dels,p):
	indx=ini+1 if forward else ini-1
	pnt=points[indx]
	puts=None
	if pnt._inter_:
		gap=indx!=ix
		if forward:
			aux=points[indx+1]
			d=[dels[len(dels)-1][1],aux]
			if gap and ini>0:
				puts=[dels[0][0],aux]
			dels.clear()
			dels.append(d)
			if gap:
				move_inter_end(forward,ix,dels)
			ix-=1
		else:
			aux=points[indx-1]
			d=[aux,dels[0][0]]
			if gap and ini<len(points)-1:
				puts=[aux,dels[len(dels)-1][1]]
			dels.clear()
			dels.append(d)
			if gap:
				move_inter_end(forward,ix,dels)
			ini-=1
		pr=pnt.get_parent()
		if pr:
			pr.remove(pnt)
		pnt._remove_(indx)
	else:
		if ini>0 and ini<len(points)-1:
			puts=[dels[0][0],dels[1][1]]
		dels.clear()
		move_inter_end(forward,ix,dels)
	del points[ini]
	points.insert(ix,p)
	pts=graph.take(ix,p)
	if puts:
		pts.append(puts)
	return pts
def move_inter_end(forward,ix,dels):
	if forward:
		if ix==len(points)-1:
			return
		dels.append([points[ix],points[ix+1]])
	elif ix>0:
		dels.append([points[ix-1],points[ix]])
fpath_js='json'
def dpath(f_in):
	p=os.path.dirname(f_in)
	return os.path.join(p,'_'+sets.pkgname+'cache_')
def fpath(f_in,ext):
	return os.path.join(dpath(f_in),os.path.basename(f_in)+'.'+ext)
def fpath_full(d_in,f_in):
	return os.path.join(d_in,os.path.basename(f_in)+'.'+fpath_js)
def write(f_in):
	p=dpath(f_in)
	f_out=fpath_full(p,f_in)
	if len(points):
		pathlib.Path(p).mkdir(exist_ok=True)#parents=False, FileExistsError exceptions will be ignored
		with open(f_out,"w") as f:
			d=[]
			for po in points:
				d.append([po._offset_,po._height_,po._inter_,po._concav_])
			a=play.wavefile
			d=[d,(a.getsampwidth(),a.getnchannels(),a.getframerate(),draw.length)]
			json.dump(d,f)
			print(f_out)
	elif os.path.exists(f_out):
		os.remove(f_out)
def read(f_in,fast):
	f_out=fpath(f_in,fpath_js)
	if not fast:
		fast=os.path.exists(f_out)
	if fast:
		with open(f_out) as f:
			if data:=f.read():
				d=json.loads(data)
				for p in d[0]:
					add(p[0],p[1],p[2],p[3],len(points))
				return d[1]

def newp(o,h,i,c):
	po=point.struct()
	po._offset_=o
	po._height_=h
	po._inter_=i
	po._concav_=c
	return po

def add(o,h,i,c,pos):
	points.insert(pos,newp(o,h,i,c))

def serialize(pnts):
	s=[]
	for p in pnts:
		s.append([p._offset_,p._height_,p._inter_,p._concav_])
	return s
def deserialize(arr):
	pnts=[]
	for a in arr:
		pnts.append(newp(a[0],a[1],a[2],a[3]))
	return pnts




from gi.repository import Gtk

from . import sets
from . import bar
from . import draw
from . import r_offset
from . import drawscroll
from . import delete
from . import play
from . import forms
from . import reload
from . import info
from . import goto

on='+'
#off='-'
control=None
char_delete=chr(0x2421)

def init():
	global start,end
	b=Gtk.Box(homogeneous=True)
	start=r_offset.inttext("0")
	start.set_halign(Gtk.Align.START)
	b.append(start)
	end=r_offset.inttext("0")
	end.set_halign(Gtk.Align.END)
	b.append(end)
	return b

def press(g,n,x,y,d):
	r_offset.calculate(int(x if drawscroll.landscape else y))

def add(a,b,i,c,lst):
	d=sets.colorButton(a,b,i,c)
	bar.box.append(d)
	lst.append(d)
	return d
def open():
	global stop,moveleft,moveright
	lst=[]
	button=add(on,toggle,"Selection Mode",draw.cont,lst)
	add(char_delete,delete.act,"Delete",None,lst)
	add(chr(0x1f58b),play.save,"Save All",None,lst)
	add(chr(0x1f58a),play.saveshort,"Save Points",None,lst)
	moveleft=add("&lt;",drawscroll.move,"Left Move (,)",False,lst)
	moveright=add("&gt;",drawscroll.move,"Right Move (.)",True,lst)
	add(chr(0x21B7),goto.open,"Go To",None,lst)
	add(chr(0x24D8),info.open,"Wave information",None,lst)
	stop=add("x",close,"Close",{'b':button,'list':lst},lst)
	#
	drawscroll.open()
	forms.open()
def close(s,d):
	b=d['b']
	if control:
		b.emit(sets._click_)
	for x in d['list']:
		bar.box.remove(x)
	reset()
	draw.close()
	drawscroll.close()
	forms.close()
	reload.close()
def reset():
	start._set_text_("0")
	end._set_text_("0")

def toggle(b,a):
	global control
	if control:
		a.remove_controller(control)
		control=None
		b._set_text_(on)
	else:
		control = Gtk.GestureClick()
		control.connect("pressed",press,None)
		a.add_controller(control)
		b._set_text_('-')
